﻿using System;


namespace WindowsFormsApplication1
{
    class TGivens
    {
        public int maxOrder;
        public double[,] m;
        public double[,] r;
        public double[] y;

        public TGivens(int size, double[,] matrix, double[] solution)
        {
            maxOrder = size;
            m = matrix;
            y = solution;
            r = new double[maxOrder, maxOrder];
        }

        public void MultMatrix(double[,] m1, int p, int q)
        {
            int i, j, k;
            double[,] ml = new double[maxOrder, maxOrder];
            double[] yl = new double[maxOrder];
            for (i = 0; i < maxOrder; i++)
            {
                if ((i == p) || (i == q))
                {
                    yl[i] = 0.0;
                    for (j = 0; j < maxOrder; j++)
                    {
                        yl[i] = yl[i] + (y[j] * m1[i, j]);
                        ml[i, j] = 0.0;
                        for (k = 0; k < maxOrder; k++)
                        {
                            ml[i, j] = ml[i, j] + (m[k, j] * m1[i, k]);
                        }
                    }
                }
                else
                {
                    yl[i] = y[i];
                    for (j = 0; j < maxOrder; j++)
                    {
                        ml[i, j] = m[i, j];
                    }
                }
            }
            for (i = 0; i < maxOrder; i++)
            {
                y[i] = yl[i];
                for (j = 0; j < maxOrder; j++)
                {
                    m[i, j] = ml[i, j];
                }
            }
        }


        public void CalcRMatrix(int p, int q)
        {
            int i, j;
            double radius;
            radius = Math.Sqrt(m[p, p] * m[p, p] + m[q, p] * m[q, p]);
            for (i = 0; i < maxOrder; i++)
            {
                for (j = 0; j < maxOrder; j++)
                {
                    r[i, j] = 0.0;
                }
                r[i, i] = 1.0;
            }
            if (radius != 0.0)
            {
                if (m[p, p] == 0.0)
                {
                    r[p, p] = 0.0;
                    r[p, q] = 1.0;
                }
                else
                {
                    r[p, p] = m[p, p] / radius;
                    r[p, q] = m[q, p] / radius;
                }
                r[q, q] = r[p, p];
                r[q, p] = -r[p, q];
            }
        }


        public void RunGivens(int maxP, int maxQ)
        {
            int i, k;
            for (i = 0; i < maxP; i++)
            {
                for (k = i + 1; k < maxQ; k++)
                {
                    if (m[k, i] != 0.0)
                    {
                        CalcRMatrix(i, k);
                        MultMatrix(r, i, k);
                    }
                }
            }
        }


        public double[] SolveRMatrix()
        {
            int k, l;
            double[] x = new double[maxOrder];
            for (k = maxOrder - 1; k >= 0; k--)
            {
                for (l = maxOrder - 1; l > k; l--)
                {
                    y[k] = y[k] - x[l] * m[k, l];
                }
                if (m[k, k] != 0)
                    x[k] = y[k] / m[k, k];
                else
                    x[k] = 0;
            }
            return x;
        }
    }
}
